@@ -1,7 +1,7 @@
 /*===========================================================================*
  * This file is part of the Abstract Library for Parallel Search (ALPS).     *
  *                                                                           *
- * ALPS is distributed under the Common Public License as part of the        *
+ * ALPS is distributed under the Eclipse Public License as part of the       *
  * COIN-OR repository (http://www.coin-or.org).                              *
  *                                                                           *
  * Authors:                                                                  *
@@ -17,7 +17,7 @@
  *          Matthew Saltzman, Clemson University                             *
  *                                                                           * 
  *                                                                           *
- * Copyright (C) 2001-2010, Lehigh University, Yan Xu, and Ted Ralphs.       *
+ * Copyright (C) 2001-2011, Lehigh University, Yan Xu, and Ted Ralphs.       *
  *===========================================================================*/
 
 #ifndef AlpsMessage_h_
@@ -54,7 +54,6 @@ enum ALPS_Message
     ALPS_MSG_HOW,
     ALPS_NODE_COUNT,
     ALPS_NODE_MEM_SIZE,
-    ALPS_P_VERSION,
     ALPS_PARAMFILE,
     ALPS_PEAK_MEMORY,
     ALPS_RAMPUP_HUB,
@@ -89,7 +88,7 @@ enum ALPS_Message
     ALPS_T_INFEASIBLE,
     ALPS_T_NODE_LIMIT,
     ALPS_T_NO_MEMORY,
-    ALPS_T_OPTIMAL,
+    ALPS_T_COMPLETE,
     ALPS_T_SOL_LIMIT,
     ALPS_T_TIME_LIMIT,
     // Following are for serial only
@@ -97,15 +96,16 @@ enum ALPS_Message
     ALPS_S_SEARCH_START,
     ALPS_S_SEARCH_SOL,
     ALPS_S_FINAL_SOL,
+    ALPS_S_FINAL_SOL_WD,
     ALPS_S_FINAL_NO_SOL,
     ALPS_S_FINAL_NODE_PROCESSED,
+    ALPS_S_FINAL_NODE_PARTIAL,
     ALPS_S_FINAL_NODE_BRANCHED,
     ALPS_S_FINAL_NODE_DISCARDED,
     ALPS_S_FINAL_NODE_LEFT,
     ALPS_S_FINAL_DEPTH,
     ALPS_S_FINAL_CPU,
     ALPS_S_FINAL_WALLCLOCK,
-    ALPS_S_VERSION,
     ALPS_WORK_UNIT,
     ALPS_DUMMY_END
 };
