@@ -21,10 +21,10 @@ namespace Bonmin
       CoinMessages((int) BON_CHOOSE_MESSAGES_DUMMY_END)
   {
     strcpy(source_,"BON");
-    ADD_MSG(PS_COST_HISTORY,std_m,5,"%3d up %3d  %15.8e  down %3d  %15.8e");
-    ADD_MSG(PS_COST_MULT,std_m, 5, "upMultiplier = %e downMultiplier = %e");
-    ADD_MSG(PS_COST_ESTIMATES, std_m, 5, "%3d value = %e upEstimate = %e downEstimate = %e infeas = %e value2 = %e");
-    ADD_MSG(CANDIDATE_LIST,std_m,4,
+    ADD_MSG(PS_COST_HISTORY,std_m,6,"%3d up %3d  %15.8e  down %3d  %15.8e");
+    ADD_MSG(PS_COST_MULT,std_m, 6, "upMultiplier = %e downMultiplier = %e");
+    ADD_MSG(PS_COST_ESTIMATES, std_m, 6, "%3d value = %e upEstimate = %e downEstimate = %e infeas = %e value2 = %e");
+    ADD_MSG(CANDIDATE_LIST,std_m,5,
         "list_[%5d] = %5d, usefull_[%5d] = %23.16e %23.16e");
     ADD_MSG(CANDIDATE_LIST2, std_m, 5,
         "list_[%3d] = %3d useful_[%3d] = %e");
@@ -48,19 +48,19 @@ namespace Bonmin
       pseudoCosts_()
   {
     jnlst_ = b.journalist();
-    SmartPtr<OptionsList> options = b.options();
+    Ipopt::SmartPtr<Ipopt::OptionsList> options = b.options();
 
     handler_ = new CoinMessageHandler;
 
-    options->GetIntegerValue("bb_log_level", bb_log_level_, "bonmin.");
+    options->GetIntegerValue("bb_log_level", bb_log_level_, b.prefix());
     handler_->setLogLevel(bb_log_level_);
-    options->GetNumericValue("time_limit", time_limit_, "bonmin.");
-    options->GetNumericValue("setup_pseudo_frac", setup_pseudo_frac_, "bonmin.");
-    options->GetNumericValue("maxmin_crit_no_sol", maxmin_crit_no_sol_, "bonmin.");
-    options->GetNumericValue("maxmin_crit_have_sol", maxmin_crit_have_sol_, "bonmin.");
-    options->GetEnumValue("trust_strong_branching_for_pseudo_cost",trustStrongForPseudoCosts_ , "bonmin.");
+    options->GetNumericValue("time_limit", time_limit_, b.prefix());
+    options->GetNumericValue("setup_pseudo_frac", setup_pseudo_frac_, b.prefix());
+    options->GetNumericValue("maxmin_crit_no_sol", maxmin_crit_no_sol_, b.prefix());
+    options->GetNumericValue("maxmin_crit_have_sol", maxmin_crit_have_sol_, b.prefix());
+    options->GetEnumValue("trust_strong_branching_for_pseudo_cost",trustStrongForPseudoCosts_ , b.prefix());
     int sortCrit;
-    options->GetEnumValue("candidate_sort_criterion", sortCrit, "bonmin.");
+    options->GetEnumValue("candidate_sort_criterion", sortCrit, b.prefix());
 #ifndef OLD_USEFULLNESS
     sortCrit_ = (CandidateSortCriterion) sortCrit;
 #endif
@@ -74,13 +74,13 @@ namespace Bonmin
     setNumberStrong(b.getIntParameter(BabSetupBase::NumberStrong));
 
     /** Get values of options specific to BonChooseVariable.*/
-    if (!options->GetIntegerValue("number_before_trust_list", numberBeforeTrustedList_, "bonmin.")) {
+    if (!options->GetIntegerValue("number_before_trust_list", numberBeforeTrustedList_, b.prefix())) {
       // default is to use the same value as for numberBeforeTrusted
       numberBeforeTrustedList_ = numberBeforeTrusted;
     }
-    options->GetIntegerValue("number_strong_branch_root", numberStrongRoot_, "bonmin.");
-    options->GetIntegerValue("min_number_strong_branch", minNumberStrongBranch_, "bonmin.");
-    options->GetIntegerValue("number_look_ahead", numberLookAhead_, "bonmin.");
+    options->GetIntegerValue("number_strong_branch_root", numberStrongRoot_, b.prefix());
+    options->GetIntegerValue("min_number_strong_branch", minNumberStrongBranch_, b.prefix());
+    options->GetIntegerValue("number_look_ahead", numberLookAhead_, b.prefix());
 
     start_time_ = CoinCpuTime();
   }
@@ -163,40 +163,41 @@ namespace Bonmin
         "most-fractional", "Sort by decreasing integer infeasibility",
         "least-fractional", "Sort by increasing integer infeasibility","");
 
-    roptions->setOptionExtraInfo("candidate_sort_criterion",31);
+    roptions->setOptionExtraInfo("candidate_sort_criterion",63);
 
     roptions->AddBoundedNumberOption("setup_pseudo_frac", "Proportion of strong branching list that has to be taken from most-integer-infeasible list.",
         0., false, 1., false, 0.5);
-    roptions->setOptionExtraInfo("setup_pseudo_frac",31);
+    roptions->setOptionExtraInfo("setup_pseudo_frac",63);
     roptions->AddBoundedNumberOption("maxmin_crit_no_sol", "Weight towards minimum in of lower and upper branching estimates when no solution has been found yet.",
         0., false, 1., false, 0.7);
-    roptions->setOptionExtraInfo("maxmin_crit_no_sol",31);
+    roptions->setOptionExtraInfo("maxmin_crit_no_sol",63);
     roptions->AddBoundedNumberOption("maxmin_crit_have_sol", "Weight towards minimum in of lower and upper branching estimates when a solution has been found.",
         0., false, 1., false, 0.1);
-    roptions->setOptionExtraInfo("maxmin_crit_have_sol",31);
+    roptions->setOptionExtraInfo("maxmin_crit_have_sol",63);
     roptions->AddLowerBoundedIntegerOption("number_before_trust_list",
         "Set the number of branches on a variable before its pseudo costs are to be believed during setup of strong branching candidate list.",
         -1, 0, "The default value is that of \"number_before_trust\"");
-    roptions->setOptionExtraInfo("number_before_trust_list",31);
+    roptions->setOptionExtraInfo("number_before_trust_list",63);
     roptions->AddLowerBoundedIntegerOption("number_strong_branch_root",
         "Maximum number of variables considered for strong branching in root node.",
         0, COIN_INT_MAX, "");
-    roptions->setOptionExtraInfo("number_strong_branch_root",31);
+    roptions->setOptionExtraInfo("number_strong_branch_root",63);
 
     roptions->AddLowerBoundedIntegerOption("min_number_strong_branch", "Sets minimum number of variables for strong branching (overriding trust)",
         0, 0,"");
-    roptions->setOptionExtraInfo("min_number_strong_branch",31);
+    roptions->setOptionExtraInfo("min_number_strong_branch",63);
     roptions->AddStringOption2("trust_strong_branching_for_pseudo_cost",
-                               "Wether or not to trust strong branching results for updating pseudo costs.",
+                               "Whether or not to trust strong branching results for updating pseudo costs.",
                                "yes",
                                "no","",
                                "yes","",
                                ""
                                );
-    roptions->setOptionExtraInfo("trust_strong_branching_for_pseudo_cost", 31);
+    roptions->setOptionExtraInfo("trust_strong_branching_for_pseudo_cost", 63);
 
     roptions->AddLowerBoundedIntegerOption("number_look_ahead", "Sets limit of look-ahead strong-branching trials",
         0, 0,"");
+    roptions->setOptionExtraInfo("number_look_ahead", 31);
   }
 
 
@@ -629,9 +630,9 @@ namespace Bonmin
             i < minNumberStrongBranch_ ||
             (
               only_pseudo_when_trusted_ && number_not_trusted_>0 ) ||
-              !isRoot && (upNumber[iObject]<numberBeforeTrusted ||
-                          downNumber[iObject]<numberBeforeTrusted )||
-              isRoot && (!upNumber[iObject] && !downNumber[iObject]) ) {
+              ( !isRoot && (upNumber[iObject]<numberBeforeTrusted ||
+                          downNumber[iObject]<numberBeforeTrusted ))||
+              ( isRoot && (!upNumber[iObject] && !downNumber[iObject])) ) {
          
              results_.push_back(HotInfo(solver, info,
                                 solver->objects(), iObject));
@@ -673,6 +674,9 @@ namespace Bonmin
           for (unsigned int i=0;i < results_.size();i++) {
             int iObject = results_[i].whichObject();
             double upEstimate;
+            if (results_[i].downStatus()== 2 || results_[i].upStatus()==2) {
+              //continue;
+            }
             if (results_[i].upStatus()!=1) {
               assert (results_[i].upStatus()>=0);
               upEstimate = results_[i].upChange();
@@ -746,7 +750,7 @@ namespace Bonmin
       }
       if ( bestObjectIndex_ >=0 ) {
         OsiObject * obj = solver->objects()[bestObjectIndex_];
-        obj->setWhichWay(	bestWhichWay_);
+        obj->setWhichWay(bestWhichWay_);
         message(BRANCH_VAR)<<obj->columnNumber()<< bestWhichWay_
         <<CoinMessageEol;
       }
@@ -808,6 +812,8 @@ namespace Bonmin
         branch->branch(solver);
         // maybe we should check bounds for stupidities here?
         solver->solveFromHotStart() ;
+
+
       } else {
         // adding cuts or something 
         thisSolver = solver->clone();
@@ -828,6 +834,10 @@ namespace Bonmin
         status0=0;
         }
       }
+      if(solver->getRowCutDebugger() && status0 == 1 ){
+           OsiTMINLPInterface * tminlp_solver = dynamic_cast<OsiTMINLPInterface *> (solver);
+           throw tminlp_solver->newUnsolvedError(1, tminlp_solver->problem(), "SB");
+      }
       numberStrongIterations_ += thisSolver->getIterationCount();
       if (solver!=thisSolver)
         delete thisSolver;
@@ -846,6 +856,7 @@ namespace Bonmin
         // ordinary
         branch->branch(solver);
         // maybe we should check bounds for stupidities here?
+        fflush(stdout);
         solver->solveFromHotStart() ;
       } else {
         // adding cuts or something 
@@ -855,6 +866,8 @@ namespace Bonmin
         int limit;
         thisSolver->getIntParam(OsiMaxNumIterationHotStart,limit);
         thisSolver->setIntParam(OsiMaxNumIteration,limit); 
+
+
         thisSolver->resolve();
       }
       // can check if we got solution
@@ -868,6 +881,10 @@ namespace Bonmin
         status1=0;
         }
       }
+      if(solver->getRowCutDebugger() && status1 == 1){
+           OsiTMINLPInterface * tminlp_solver = dynamic_cast<OsiTMINLPInterface *> (solver);
+           throw tminlp_solver->newUnsolvedError(1, tminlp_solver->problem(), "SB");
+      }
       numberStrongIterations_ += thisSolver->getIterationCount();
       if (solver!=thisSolver)
         delete thisSolver;
@@ -1015,8 +1032,9 @@ BonChooseVariable::updateInformation( int index, int branch,
   if(cbc_model_ == NULL) return;
   assert (index<solver_->numberObjects()); 
   assert (branch<2); 
-  assert (changeInValue>0.0); 
-  assert (branch<2); 
+
+  if(fabs(changeInValue) < 1e-6) return;
+
   double* upTotalChange = pseudoCosts_.upTotalChange(); 
   double* downTotalChange = pseudoCosts_.downTotalChange(); 
   int* upNumber = pseudoCosts_.upNumber(); 
@@ -1107,7 +1125,19 @@ BonChooseVariable::updateInformation( int index, int branch,
     for(int i = 0 ; i < numObject ; i++){
        infeasibility += objects[i]->checkInfeasibility(info);
     }
-    return OsiHotInfo::updateInformation(solver, info, choose);
+    int status = OsiHotInfo::updateInformation(solver, info, choose);
+#if 1
+    if(!solver->isProvenPrimalInfeasible() && !solver->isProvenOptimal()){
+      status = 2;
+      statuses_[iBranch] = status;
+    }
+    else if(solver->isProvenPrimalInfeasible() && fabs(solver->getObjValue()) < 1e-06) {
+      fprintf(stderr, "Very small infeasibility: %g\n", solver->getObjValue());
+      status = 2;
+      statuses_[iBranch] = status;
+    }
+#endif
+    return status;
   }
   
 }/* Ends Bonmin's namespace.*/
