@@ -2,7 +2,7 @@
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: BonCurvatureEstimator.cpp 1313 2008-08-22 22:36:24Z pbonami $
+// $Id: BonCurvatureEstimator.cpp 1757 2010-12-09 07:21:18Z pbonami $
 //
 // Author:   Andreas Waechter                 IBM    2006-10-11
 
@@ -32,7 +32,7 @@
 #ifdef HAVE_WSMP
 # include "IpWsmpSolverInterface.hpp"
 #endif
-#ifdef HAVE_MUMPS
+#ifdef COIN_HAS_MUMPS
 # include "IpMumpsSolverInterface.hpp"
 #endif
 
@@ -136,7 +136,7 @@ namespace Bonmin
 
     }
     else if (linear_solver=="mumps") {
-#ifdef HAVE_MUMPS
+#ifdef COIN_HAS_MUMPS
       SolverInterface1 = new MumpsSolverInterface();
       SolverInterface2 = new MumpsSolverInterface();
 #else
