@@ -1,6 +1,8 @@
-/* $Id: CbcStrategy.cpp 1521 2010-11-04 16:16:21Z lou $ */
+/* $Id: CbcStrategy.cpp 1675 2011-06-19 17:23:14Z stefan $ */
 // Copyright (C) 2005, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #if defined(_MSC_VER)
 // Turn off compiler warning about long names
 #  pragma warning(disable:4786)
@@ -39,6 +41,7 @@
 
 #include "CbcHeuristic.hpp"
 #include "CbcHeuristicLocal.hpp"
+#include "CbcHeuristicRINS.hpp"
 
 // Default Constructor
 CbcStrategy::CbcStrategy()
@@ -417,8 +420,8 @@ CbcStrategyDefault::setupOther(CbcModel & model)
                 }
             }
             if (!numberNon && numberClose) {
-                printf("Tidying %d multiples of 0.01, %d close\n",
-                       numberInt, numberClose);
+                COIN_DETAIL_PRINT(printf("Tidying %d multiples of 0.01, %d close\n",
+					 numberInt, numberClose));
                 for (iColumn = 0; iColumn < numberColumns; iColumn++) {
                     for (int j = columnStart[iColumn];
                             j < columnStart[iColumn] + columnLength[iColumn]; j++) {
@@ -922,6 +925,40 @@ CbcStrategyDefaultSubTree::setupHeuristics(CbcModel & model)
     }
     if (!found)
         model.addHeuristic(&heuristic1);
+    if ((model.moreSpecialOptions()&32768)!=0) {
+      // Allow join solutions
+      CbcHeuristicLocal heuristic2(model);
+      heuristic2.setHeuristicName("join solutions");
+      //sheuristic2.setSearchType(1);
+      found = false;
+      for (iHeuristic = 0; iHeuristic < numberHeuristics; iHeuristic++) {
+        CbcHeuristic * heuristic = model.heuristic(iHeuristic);
+        CbcHeuristicLocal * cgl = dynamic_cast<CbcHeuristicLocal *>(heuristic);
+        if (cgl) {
+	  found = true;
+	  break;
+        }
+      }
+      if (!found)
+        model.addHeuristic(&heuristic2);
+      // Allow RINS
+      CbcHeuristicRINS heuristic5(model);
+      heuristic5.setHeuristicName("RINS");
+      heuristic5.setFractionSmall(0.5);
+      heuristic5.setDecayFactor(5.0);
+      //heuristic5.setSearchType(1);
+      found = false;
+      for (iHeuristic = 0; iHeuristic < numberHeuristics; iHeuristic++) {
+        CbcHeuristic * heuristic = model.heuristic(iHeuristic);
+        CbcHeuristicLocal * cgl = dynamic_cast<CbcHeuristicLocal *>(heuristic);
+        if (cgl) {
+	  found = true;
+	  break;
+        }
+      }
+      if (!found)
+        model.addHeuristic(&heuristic5);
+    }
 }
 // Do printing stuff
 void
