@@ -1,29 +1,29 @@
-/* $Id: CouenneObject.cpp 141 2009-06-03 04:19:19Z pbelotti $ */
-/*
+/* $Id: CouenneObject.cpp 742 2011-07-22 19:19:09Z pbelotti $
+ *
  * Name:    CouenneObject.cpp
  * Authors: Pierre Bonami, IBM Corp.
  *          Pietro Belotti, Carnegie Mellon University
  * Purpose: Base object for variables (to be used in branching)
  *
- * (C) Carnegie-Mellon University, 2006-09.
- * This file is licensed under the Common Public License (CPL)
+ * (C) Carnegie-Mellon University, 2006-10.
+ * This file is licensed under the Eclipse Public License (EPL)
  */
 
-#include "CoinHelperFunctions.hpp"
-#include "CoinFinite.hpp"
-
 #include "CouenneProblem.hpp"
+#include "CouenneProblemElem.hpp"
 #include "CouenneObject.hpp"
 #include "CouenneBranchingObject.hpp"
 
-const CouNumber default_alpha  = 0.2;
-const CouNumber default_clamp  = 0.2;
-const CouNumber max_pseudocost = 1000.;
+#include "CoinHelperFunctions.hpp"
+
+using namespace Ipopt;
+using namespace Couenne;
 
 /// Empty constructor
 CouenneObject::CouenneObject ():
 
   OsiObject (),
+  cutGen_          (NULL),
   problem_         (NULL),
   reference_       (NULL),
   strategy_        (MID_INTERVAL),
@@ -39,11 +39,13 @@ CouenneObject::CouenneObject ():
 
 
 /// Constructor with information for branching point selection strategy
-CouenneObject::CouenneObject (CouenneProblem *p, 
+CouenneObject::CouenneObject (CouenneCutGenerator *cutgen,
+			      CouenneProblem *p, 
 			      exprVar *ref, 
 			      Bonmin::BabSetupBase *base, 
 			      JnlstPtr jnlst):
   OsiObject (),
+  cutGen_         (cutgen),
   problem_        (p),  
   reference_      (ref),
   strategy_       (MID_INTERVAL),
@@ -91,6 +93,7 @@ CouenneObject::CouenneObject (exprVar *ref,
 			      JnlstPtr jnlst):
 
   OsiObject (),
+  cutGen_         (NULL),  
   problem_        (NULL),  
   reference_      (ref),
   strategy_       (MID_INTERVAL),
@@ -112,6 +115,7 @@ CouenneObject::CouenneObject (exprVar *ref,
 /// Copy constructor
 CouenneObject::CouenneObject (const CouenneObject &src):
   OsiObject       (src),
+  cutGen_         (src.cutGen_),
   problem_        (src.problem_),
   reference_      (src.reference_),
   strategy_       (src.strategy_),
@@ -126,6 +130,20 @@ CouenneObject::CouenneObject (const CouenneObject &src):
   pseudoMultType_ (src.pseudoMultType_) {}
 
 
+/// integer infeasibility: min {value - floor(value), ceil(value) -
+/// value}. Handle special cases so that Cbc and Couenne agree. Also,
+/// check out-of-bounds values.
+
+double CouenneObject::intInfeasibility (double value, double lb, double ub) const {
+
+  if      (value < lb) value = lb;
+  else if (value > ub) value = ub;
+
+  return CoinMin (value - floor (value + COUENNE_EPS), 
+		          ceil  (value - COUENNE_EPS) - value);
+}
+
+
 /// apply the branching rule
 OsiBranchingObject *CouenneObject::createBranch (OsiSolverInterface *si,
 						 const OsiBranchingInformation *info,
@@ -149,7 +167,8 @@ OsiBranchingObject *CouenneObject::createBranch (OsiSolverInterface *si,
     (problem_ -> nVars (),
      info -> solution_,
      info -> lower_,
-     info -> upper_);
+     info -> upper_,
+     false);
 
   CouNumber  
     *brPts  = NULL,         // branching point(s)
@@ -165,8 +184,11 @@ OsiBranchingObject *CouenneObject::createBranch (OsiSolverInterface *si,
       selectBranch (this, info,                      // input parameters
 		    brVar, brPts, brDist, whichWay); // result: who, where, distance, and direction
   else {
+
+    // reference_ -> Image () == NULL implies this is an integer object
+
     brVar = reference_;
-    brPts  = (double *) realloc (brPts, sizeof (double)); 
+    brPts  = (double *) realloc (brPts,      sizeof (double)); 
     brDist = (double *) realloc (brDist, 2 * sizeof (double)); 
 
     double point = info -> solution_ [reference_ -> Index ()];
@@ -229,7 +251,7 @@ OsiBranchingObject *CouenneObject::createBranch (OsiSolverInterface *si,
 
   // create branching object /////////////////////////////////////// 
   OsiBranchingObject *brObj = new CouenneBranchingObject 
-    (si, this, jnlst_, brVar, way, *brPts, doFBBT_, doConvCuts_);
+    (si, this, jnlst_, cutGen_, problem_, brVar, way, *brPts, doFBBT_, doConvCuts_);
 
   problem_ -> domain () -> pop (); // Couenne discards current point
 
@@ -241,7 +263,47 @@ OsiBranchingObject *CouenneObject::createBranch (OsiSolverInterface *si,
 
 
 /// computes a not-too-bad point where to branch, in the "middle" of an interval
-CouNumber CouenneObject::midInterval (CouNumber x, CouNumber l, CouNumber u) const {
+CouNumber CouenneObject::midInterval (CouNumber x, CouNumber l, CouNumber u, 
+				      const OsiBranchingInformation *info) const {
+
+  CouNumber curAlpha = alpha_;
+
+#if 1
+  if (info) {
+
+    // adaptive scheme: make LP point count more when gap approaches zero
+    int objInd = problem_ -> Obj (0) -> Body () -> Index ();
+
+    double
+      lb = info -> lower_ [objInd],
+      ub = problem_ -> getCutOff (),
+      currentGap = 
+      (ub >  COUENNE_INFINITY    / 10 ||
+       lb < -Couenne_large_bound / 10) ? 1.e3 : 
+      fabs (ub - lb) / (1.e-3 + CoinMin (fabs (ub), fabs (lb)));
+
+#if 1
+    if (currentGap < 1e-3) {
+
+      currentGap *= 1e3;
+
+      assert ((currentGap >= 0.) && 
+	      (currentGap <= 1.));
+
+      curAlpha = currentGap * alpha_ + (1 - currentGap);
+    }
+#else
+    // make curAlpha closer to 1 by adding remaining (1-alpha_)
+    // weighted inversely proportional to gap
+    curAlpha += (1 - alpha_) / (1. + currentGap);
+#endif
+
+    // printf ("using %g rather than %g. cutoff %g, lb %g, gap %g\n", 
+    // 	    curAlpha, alpha_,
+    // 	    problem_ -> getCutOff (),
+    // 	    lb, currentGap);
+  }
+#endif
 
   if (u < l + COUENNE_EPS)
     return (0.5 * (l + u));
@@ -249,30 +311,34 @@ CouNumber CouenneObject::midInterval (CouNumber x, CouNumber l, CouNumber u) con
   if      (x<l) x = l;
   else if (x>u) x = u;
 
-  if   (l < -COUENNE_INFINITY / 10)
-    if (u >  COUENNE_INFINITY / 10) return x; // 0.                                    // ]-inf,+inf[
-    else                            return ((x < -COUENNE_EPS) ? (AGGR_MUL * (-1+x)) : // ]-inf,u]
-					    (x >  COUENNE_EPS) ? 0. : -AGGR_MUL);
+  if   (l < -large_bound)
+    if (u >  COUENNE_EPS) return 0.;                                        // ]-inf,+inf[
+    else                  return CoinMax ((l+u)/2, (AGGR_MUL * (-1. + u))); // ]-inf,u]
   else
-    if (u >  COUENNE_INFINITY / 10) return ((x >  COUENNE_EPS) ? (AGGR_MUL *  (1+x)) : // [l,+inf[
-					    (x < -COUENNE_EPS) ? 0. :  AGGR_MUL);
-    else {                                                                             // [l,u]
-      CouNumber point = alpha_ * x + (1. - alpha_) * (l + u) / 2.;
+    if (u >  large_bound)                                                   // [l,+inf[
+      if (l < - COUENNE_EPS) return 0.;
+      else                   return CoinMin ((l+u)/2, (AGGR_MUL * (1. + l)));
+    else {                                                                  // [l,u]
+
+      CouNumber point = curAlpha * x + (1. - curAlpha) * (l + u) / 2.;
+
       if      ((point-l) / (u-l) < closeToBounds) point = l + (u-l) * closeToBounds;
       else if ((u-point) / (u-l) < closeToBounds) point = u + (l-u) * closeToBounds;
+
       return point;
     }
 }
 
 
 /// pick branching point based on current strategy
-CouNumber CouenneObject::getBrPoint (funtriplet *ft, CouNumber x0, CouNumber l, CouNumber u) const {
+CouNumber CouenneObject::getBrPoint (funtriplet *ft, CouNumber x0, CouNumber l, CouNumber u, 
+				     const OsiBranchingInformation *info) const {
 
   if ((l < -COUENNE_EPS) && 
       (u >  COUENNE_EPS) && 
       (-l/u >= THRES_ZERO_SYMM) &&
       (-u/l >= THRES_ZERO_SYMM))
-    return 0.;
+    return 0.; // this happens when [l,u] is quite symmetrical w.r.t. 0
 
   CouNumber width = lp_clamp_ * (u-l);
 
@@ -282,7 +348,8 @@ CouNumber CouenneObject::getBrPoint (funtriplet *ft, CouNumber x0, CouNumber l,
   case CouenneObject::BALANCED:     return minMaxDelta (ft, l, u);
   case CouenneObject::LP_CLAMPED:   return CoinMax (l + width, CoinMin (x0, u - width));
   case CouenneObject::LP_CENTRAL:   return ((x0 < l + width) || (x0 > u - width)) ? (l+u)/2 : x0;
-  case CouenneObject::MID_INTERVAL: return midInterval (x0, l, u);
+  case CouenneObject::MID_INTERVAL: return midInterval (x0, l, u, info);
+
   default:
     printf ("Couenne: unknown branching point selection strategy\n");
     exit (-1);
@@ -300,30 +367,36 @@ double CouenneObject::infeasibility (const OsiBranchingInformation *info, int &w
     (problem_ -> nVars (),
      info -> solution_, 
      info -> lower_, 
-     info -> upper_);
+     info -> upper_,
+     false);
 
   double retval = checkInfeasibility (info);
 
   problem_ -> domain () -> pop ();
 
+  bool isInt = reference_ -> isInteger ();
+
+  int refInd = reference_ -> Index ();
+  double point = info -> solution_ [refInd];
+
   if (pseudoMultType_ == INFEASIBILITY) {
 
-    double point = info -> solution_ [reference_ -> Index ()];
+    if (isInt) {
+      CouNumber intInfeas = intInfeasibility (point, info -> lower_ [refInd], info -> upper_ [refInd]);
 
-    if (reference_ -> isInteger ()) {
-      if (retval < intInfeasibility (point)) {
+      if (retval < intInfeas) {
 	if (downEstimate_ <       point  - floor (point)) downEstimate_ =       point  - floor (point);
 	if (upEstimate_   < ceil (point) -        point)  upEstimate_   = ceil (point) -        point;
-	retval = intInfeasibility (point);
+	retval = intInfeas;
       }
     }
     else upEstimate_ = downEstimate_ = retval;
   }
   else setEstimates (info, &retval, NULL);
 
-  return (reference_ -> isInteger ()) ? 
-    CoinMax (retval, intInfeasibility (info -> solution_ [reference_ -> Index ()])) :
-    retval;
+  return (isInt ? 
+    CoinMax (retval, intInfeasibility (point, info -> lower_ [refInd], info -> upper_ [refInd])) :
+    retval);
 }
 
 
@@ -332,9 +405,13 @@ double CouenneObject::infeasibility (const OsiBranchingInformation *info, int &w
 /// CouenneVarObject::infeasibility()
 double CouenneObject::checkInfeasibility (const OsiBranchingInformation *info) const {
 
+  int refInd = reference_ -> Index ();  
+
   if (reference_ -> Type () == VAR)
     return (reference_ -> isInteger ()) ? 
-      intInfeasibility (info -> solution_ [reference_ -> Index ()]) : 0.;
+      intInfeasibility (info -> solution_ [refInd],
+			info -> lower_    [refInd],
+			info -> upper_    [refInd]) : 0.;
 
   double 
     vval = info -> solution_ [reference_ -> Index ()],
@@ -351,11 +428,14 @@ double CouenneObject::checkInfeasibility (const OsiBranchingInformation *info) c
     fval = COUENNE_INFINITY;
 
   double
-    retval = fabs (vval - fval),
+    retval = 
+    ((reference_ -> sign () == expression::AUX_GEQ) && (vval >= fval)) ? 0. : 
+    ((reference_ -> sign () == expression::AUX_LEQ) && (vval <= fval)) ? 0. : fabs (vval - fval),
+
     ratio = (CoinMax (1., fabs (vval)) / 
 	     CoinMax (1., fabs (fval)));
 
-  //printf ("checkinf --> v=%e f=%e den=%e ret=%e ratio=%e\n", vval, fval, denom, retval, ratio);
+  //jnlst_ -> Printf (J_DETAILED, J_BRANCHING, "checkinf --> v=%e f=%e den=%e ret=%e ratio=%e\n", vval, fval, denom, retval, ratio);
 
   if ((ratio < 2)  && 
       (ratio > .5) &&
@@ -379,7 +459,9 @@ double CouenneObject::checkInfeasibility (const OsiBranchingInformation *info) c
     retval = 1.e20;
 
   return (reference_ -> isInteger ()) ? 
-    CoinMax (retval, intInfeasibility (info -> solution_ [reference_ -> Index ()])) :
+    CoinMax (retval, intInfeasibility (info -> solution_ [refInd],
+				       info -> lower_    [refInd],
+				       info -> upper_    [refInd])) :
     retval;
 }
 
@@ -418,6 +500,10 @@ void CouenneObject::setParameters (Bonmin::BabSetupBase *base) {
     base -> options () -> GetNumericValue ("branch_midpoint_alpha", alpha_, "couenne.");
   }
 
+  if (strategy_ == LP_CLAMPED || 
+      strategy_ == LP_CENTRAL)
+    base -> options () -> GetNumericValue ("branch_lp_clamp", lp_clamp_, "couenne.");
+
   // accept options for branching rules specific to each operator
   if (reference_ && reference_ -> Type () == AUX) {
 
@@ -455,10 +541,14 @@ void CouenneObject::setParameters (Bonmin::BabSetupBase *base) {
     if (br_operator != "") {
       // read option
       char select [40], sel_clamp [40];
+      double lp_clamp_fun = default_clamp;
       sprintf (select,    "branch_pt_select_%s", br_operator.c_str ());
       sprintf (sel_clamp, "branch_lp_clamp_%s",  br_operator.c_str ());
       base -> options () -> GetStringValue (select, brtype, "couenne.");
-      base -> options () -> GetNumericValue (sel_clamp, lp_clamp_, "couenne.");
+      base -> options () -> GetNumericValue (sel_clamp, lp_clamp_fun, "couenne.");
+
+      if (lp_clamp_fun != default_clamp)
+	lp_clamp_ = lp_clamp_fun;
 
       if      (brtype == "balanced")    strategy_ = BALANCED;
       else if (brtype == "lp-clamped")  strategy_ = LP_CLAMPED;
@@ -467,7 +557,10 @@ void CouenneObject::setParameters (Bonmin::BabSetupBase *base) {
       else if (brtype == "no-branch")   strategy_ = NO_BRANCH;
       else if (brtype == "mid-point") {
 	strategy_ = MID_INTERVAL;
-	base -> options () -> GetNumericValue ("branch_midpoint_alpha", alpha_, "couenne.");
+	double alpha_fun = default_alpha;
+	base -> options () -> GetNumericValue ("branch_midpoint_alpha", alpha_fun, "couenne.");
+	if (alpha_fun != default_alpha)
+	  alpha_ = alpha_fun;
       }
     }
   }
@@ -481,12 +574,19 @@ void CouenneObject::setEstimates (const OsiBranchingInformation *info,
 
   int index = reference_ -> Index ();
 
+  //bool isInteger = reference_ -> isInteger ();
+
   CouNumber 
     *up   = &upEstimate_,
     *down = &downEstimate_,
-     point = 0.;
+     point = 0.,
+     lower = info -> lower_ [index],
+     upper = info -> upper_ [index];
 
   ////////////////////////////////////////////////////////////
+  //
+  // these rules invert the interval, so we just flip the pointers
+
   if ((pseudoMultType_ == INTERVAL_LP_REV) ||
       (pseudoMultType_ == INTERVAL_BR_REV)) {
 
@@ -507,18 +607,25 @@ void CouenneObject::setEstimates (const OsiBranchingInformation *info,
     
     point = *brpoint;
 
-  // now move it away from the bounds
+  //printf ("point = %g [%s]\n", point, reference_ -> isInteger () ? "int" : "frac");
 
-  point = midInterval (point, 
-		       info -> lower_ [index],
-		       info -> upper_ [index]);
+  // now move it away from the bounds, unless this is an integer variable
 
-  CouNumber delta = closeToBounds * (info -> upper_ [index] - info -> lower_ [index]);
+  //if (!isInteger)
+  point = midInterval (point, lower, upper, info);
 
-  if      (point < info -> lower_ [index] + delta)
-    point        = info -> lower_ [index] + delta;
-  else if (point > info -> upper_ [index] - delta)
-    point        = info -> upper_ [index] - delta;
+  //printf ("point = %g\n", point);
+
+  if ((lower > -COUENNE_INFINITY) &&
+      (upper <  COUENNE_INFINITY)) {
+
+    CouNumber delta = closeToBounds * (upper - lower);
+
+    if      (point < lower + delta)
+      point        = lower + delta;
+    else if (point > upper - delta)
+      point        = upper - delta;
+  }
 
   ///////////////////////////////////////////////////////////
   switch (pseudoMultType_) {
@@ -533,8 +640,8 @@ void CouenneObject::setEstimates (const OsiBranchingInformation *info,
   case INTERVAL_BR:
   case INTERVAL_BR_REV:
     assert (info);
-    *up   = CoinMin (max_pseudocost,         info -> upper_ [index] - point);
-    *down = CoinMin (max_pseudocost, point - info -> lower_ [index]);
+    *up   = CoinMin (max_pseudocost, 1e-3 + fabs (upper - point));
+    *down = CoinMin (max_pseudocost, 1e-3 +         fabs (point - lower));
     break;
 
   case PROJECTDIST: // taken care of in selectBranch procedure
@@ -545,15 +652,23 @@ void CouenneObject::setEstimates (const OsiBranchingInformation *info,
     exit (-1);
   }
 
-//   if (downEstimate_ <= 0.0 || upEstimate_ <= 0.0)
-//     printf ("%g [%g,%g] ---> [%g,%g]\n", 
-// 	    point,
-// 	    info -> lower_ [index],
-// 	    info -> upper_ [index],
-// 	    downEstimate_, upEstimate_);
+  /*if (downEstimate_ <= 0.0 || upEstimate_ <= 0.0)
+     printf ("%g [%g,%g] ---> [%g,%g]\n", 
+ 	    point,
+ 	    lower,
+ 	    upper,
+ 	    downEstimate_, upEstimate_);*/
 
-  if (reference_ -> isInteger ()) {
-    if (downEstimate_ <       point  - floor (point)) downEstimate_ =       point  - floor (point);
-    if (upEstimate_   < ceil (point) -        point)  upEstimate_   = ceil (point) -        point;
-  }
+  /*if (reference_ -> isInteger ()) {
+
+    CouNumber 
+      fracDn =       point  - floor (point),
+      fracUp = ceil (point) -        point;
+
+    if (downEstimate_ < fracDn) downEstimate_ = fracDn;
+    if (upEstimate_   < fracUp)   upEstimate_ = fracUp;
+    }*/
+
+  assert (downEstimate_ > 0. &&
+          upEstimate_   > 0.);
 }
