@@ -1,19 +1,32 @@
-/* $Id: generateCuts.cpp 141 2009-06-03 04:19:19Z pbelotti $
+/* $Id: generateCuts.cpp 736 2011-07-04 02:27:17Z pbelotti $
  *
  * Name:    generateCuts.cpp
  * Author:  Pietro Belotti
  * Purpose: the generateCuts() method of the convexification class CouenneCutGenerator
  *
- * (C) Carnegie-Mellon University, 2006-08.
- * This file is licensed under the Common Public License (CPL)
+ * (C) Carnegie-Mellon University, 2006-11.
+ * This file is licensed under the Eclipse Public License (EPL)
  */
 
-#include "BonAuxInfos.hpp"
+#include "BonCbc.hpp"
+#include "BonBabInfos.hpp"
 #include "CglCutGenerator.hpp"
 
 #include "CouenneCutGenerator.hpp"
 #include "CouenneProblem.hpp"
-#include "CouenneSolverInterface.hpp"
+#include "CouenneProblemElem.hpp"
+#include "CouenneExprVar.hpp"
+#include "CouenneInfeasCut.hpp"
+
+#include "CouenneRecordBestSol.hpp"
+
+#ifdef COIN_HAS_NTY
+#include "Nauty.h"
+#endif
+
+using namespace Ipopt;
+
+namespace Couenne {
 
 #define Couenne_large_bound2 9.99e12
 
@@ -79,6 +92,41 @@ void updateBranchInfo (const OsiSolverInterface &si, CouenneProblem *p,
 void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
 					OsiCuts &cs, 
 					const CglTreeInfo info) const {
+
+  // check if out of time or if an infeasibility cut (iis of type 0)
+  // was added as a result of, e.g., pruning on BT. If so, no need to
+  // run this.
+
+  if (isWiped (cs) || 
+     (CoinCpuTime () > problem_ -> getMaxCpuTime ()))
+    return;
+
+#ifdef FM_TRACE_OPTSOL
+  double currCutOff = problem_->getCutOff();
+  double bestVal = 1e50;
+  CouenneRecordBestSol *rs = problem_->getRecordBestSol();
+  if(rs->getHasSol()) {
+    bestVal = rs->getVal(); 
+  }
+  if(currCutOff > bestVal) {
+    //problem_ -> setCutOff (bestVal - 1e-6); // FIXME: don't add numerical constants
+    problem_ -> setCutOff (bestVal);
+    OsiColCut *objCut = new OsiColCut;
+    int indObj = problem_->Obj(0)->Body()->Index();
+    objCut->setUbs(1, &indObj, &bestVal);
+    cs.insert(objCut);
+    delete objCut;
+  }
+#endif
+
+#ifdef FM_PRINT_INFO
+  if((BabPtr_ != NULL) && (info.level >= 0) && (info.pass == 0) && 
+     (BabPtr_->model().getNodeCount() > lastPrintLine)) {
+    printLineInfo();
+    lastPrintLine += 1;
+  }
+#endif
+
   const int infeasible = 1;
 
   int nInitCuts = cs.sizeRowCuts ();
@@ -169,7 +217,8 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
     // run AFTER initial FBBT...
     if (problem_ -> doFBBT () &&
 	(! (problem_ -> boundTightening (chg_bds, babInfo))))
-      printf ("Couenne: WARNING, first convexification is infeasible\n");
+          jnlst_ -> Printf (J_STRONGWARNING, J_CONVEXIFYING,
+            "Couenne: WARNING, first convexification is infeasible\n");
 
     // For each auxiliary variable replacing the original (nonlinear)
     // constraints, check if corresponding bounds are violated, and
@@ -179,6 +228,9 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
 
     for (int i=0; i<nnlc; i++) {
 
+      if (CoinCpuTime () > problem_ -> getMaxCpuTime ())
+	break;
+
       // for each constraint
       CouenneConstraint *con = problem_ -> Con (i);
 
@@ -267,14 +319,8 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
     // use new optimum as lower bound for variable associated w/objective
     int indobj = problem_ -> Obj (0) -> Body () -> Index ();
 
-    assert (indobj >= 0);
-
     // transmit solution from OsiSolverInterface to problem
-    problem_ -> domain () -> push 
-      (problem_ -> nVars (),
-       si. getColSolution (), 
-       si. getColLower    (),
-       si. getColUpper    ());
+    problem_ -> domain () -> push (&si, &cs);
 
     if (indobj >= 0) {
 
@@ -304,7 +350,8 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
 	 (con -> Body () -> Type () == VAR))) {
 
       // if there exists violation, add constraint
-      CouNumber l = con -> Lb () -> Value (),	
+      CouNumber 
+	l = con -> Lb () -> Value (),	
 	u = con -> Ub () -> Value ();
 
       // tighten bounds in Couenne's problem representation
@@ -327,14 +374,26 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
 
   try {
 
+    // Before bound tightening, compute symmetry group. After bound
+    // tightening is done, we can apply further tightening using orbit
+    // information.
+
+#ifdef COIN_HAS_NTY
+    //    ChangeBounds (psi -> getColLower (),  
+    //		  psi -> getColUpper (), 
+    //		  psi -> getNumCols ());
+    if (problem_ -> orbitalBranching ())
+      problem_ -> Compute_Symmetry ();
+#endif
+
     // Bound tightening ////////////////////////////////////
 
     /*printf ("== BT ================\n");
-    for (int i = 0; i < problem_ -> nVars (); i++)
+      for (int i = 0; i < problem_ -> nVars (); i++)
       if (problem_ -> Var (i) -> Multiplicity () > 0)
-	printf ("%4d %+20.8g [%+20.8g,%+20.8g]\n", i,
-		problem_ -> X  (i), problem_ -> Lb (i), problem_ -> Ub (i));
-		printf("=============================\n");*/
+      printf ("%4d %+20.8g [%+20.8g,%+20.8g]\n", i,
+      problem_ -> X  (i), problem_ -> Lb (i), problem_ -> Ub (i));
+      printf("=============================\n");*/
 
     // Reduced Cost BT -- to be done first to use rcost correctly
     if (!firstcall_  &&                         // have a linearization already
@@ -369,31 +428,100 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
       jnlst_->Printf(J_VECTOR, J_CONVEXIFYING,"=============================\n");
     }
 
+    // Use orbit info to tighten bounds
+
+#ifdef COIN_HAS_NTY
+
+    // TODO: when independent bound tightener, can get original bounds
+    // through si.getCol{Low,Upp}er()
+
+    if (problem_ -> orbitalBranching () && !firstcall_) {
+
+      CouNumber 
+	*lb = problem_ -> Lb (),
+	*ub = problem_ -> Ub ();
+
+      std::vector<std::vector<int> > *new_orbits = problem_ -> getNtyInfo () -> getOrbits();
+
+      for (int i=0, ii = problem_ -> getNtyInfo () -> getNumOrbits (); ii--; i++){
+
+	CouNumber
+	  ll = -COUENNE_INFINITY,
+	  uu =  COUENNE_INFINITY; 
+	
+	std::vector <int> orbit = (*new_orbits)[i];
+
+	if (orbit.size () <= 1)
+	  continue; // not much to do when only one variable in this orbit
+
+	if (jnlst_ -> ProduceOutput (J_VECTOR, J_BOUNDTIGHTENING)) {
+	  printf ("orbit bounds: "); fflush (stdout);
+	  for(int j = 0; j < orbit.size (); j++) {
+	    printf ("x_%d [%g,%g] ", orbit[j], lb [orbit [j]], ub [orbit [j]]);
+	    fflush (stdout);
+	  }
+	  printf ("\n");
+	}
+
+	for (int j = 0; j < orbit.size (); j++) {
+ 
+	  int indOrb = orbit [j];
+
+	  if (indOrb < problem_ -> nVars ()) {
+
+	    if (lb [indOrb] > ll) ll = lb [indOrb];
+	    if (ub [indOrb] < uu) uu = ub [indOrb];
+	  }
+	}
+
+	jnlst_ -> Printf (J_VECTOR, J_BOUNDTIGHTENING, 
+			  " --> new common lower bounds: [%g,--]\n", ll);
+
+	for(int j = 0; j < orbit.size (); j++) {
+
+	  int indOrb = orbit [j];
+
+	  if (indOrb < problem_ -> nVars ()){
+
+	    lb [indOrb] = ll;
+	    ub [indOrb] = uu;
+	  }
+	}
+      }
+
+      delete new_orbits;
+    }
+
+#endif
+
     // Generate convexification cuts //////////////////////////////
 
     sparse2dense (ncols, chg_bds, changed, nchanged);
 
-    double *nlpSol;
+    double *nlpSol = NULL;
 
     //--------------------------------------------
 
-    if (babInfo && ((nlpSol = const_cast <double *> (babInfo -> nlpSolution ())))) {
+    if (true) {
+
+      if (babInfo) 
+	nlpSol = const_cast <double *> (babInfo -> nlpSolution ());
 
       // Aggressive Bound Tightening ////////////////////////////////
 
       int logAbtLev = problem_ -> logAbtLev ();
 
-      if (problem_ -> doABT () &&           // flag is checked, AND
+      if (problem_ -> doABT () &&             // flag is checked, AND
 	  ((logAbtLev != 0) ||                // (parameter is nonzero OR
 	   (info.level == 0)) &&              //  we are at root node), AND
-	  (info.pass == 0) &&               // at first round of cuts, AND 
+	  (info.pass == 0) &&                 // at first round of cuts, AND 
 	  ((logAbtLev < 0) ||                 // (logAbtLev = -1, OR
 	   (info.level <= logAbtLev) ||       //  depth is lower than COU_OBBT_CUTOFF_LEVEL, OR
 	   (CoinDrand48 () <                  //  probability inversely proportional to the level)
 	    pow (2., (double) logAbtLev - (info.level + 1))))) {
 
-	jnlst_ -> Printf(J_ITERSUMMARY, J_CONVEXIFYING,"  performing ABT\n");
-	if (! (problem_ -> aggressiveBT (nlp_, chg_bds, babInfo)))
+	jnlst_ -> Printf(J_VECTOR, J_BOUNDTIGHTENING,"  performing ABT\n");
+	if (! (problem_ -> aggressiveBT (nlp_, chg_bds, info, babInfo)))
 	  throw infeasible;
 
 	sparse2dense (ncols, chg_bds, changed, nchanged);
@@ -417,10 +545,12 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
 	 problem_ -> domain () -> ub (), false);
 
       // fill originals with nlp values
-      CoinCopyN (nlpSol, problem_ -> nOrigVars (), problem_ -> domain () -> x ());
+      if (nlpSol) {
+	CoinCopyN (nlpSol, problem_ -> nOrigVars (), problem_ -> domain () -> x ());
       //problem_ -> initAuxs ();
 
       problem_ -> getAuxs (problem_ -> domain () -> x ());
+      }
 
       if (jnlst_ -> ProduceOutput (J_VECTOR, J_CONVEXIFYING)) {
 	jnlst_ -> Printf(J_VECTOR, J_CONVEXIFYING,"== genrowcuts on NLP =============\n");
@@ -441,7 +571,9 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
       addviolated_ = save_av;     // restore previous value
 
       //    if (!firstcall_) // keep solution if called from extractLinearRelaxation()
-      babInfo -> setHasNlpSolution (false); // reset it after use //AW HERE
+      if (babInfo) 
+	babInfo -> setHasNlpSolution (false); // reset it after use //AW HERE
+
     } else {
 
       if (jnlst_ -> ProduceOutput (J_VECTOR, J_CONVEXIFYING)) {
@@ -466,10 +598,10 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
       jnlst_->Printf(J_ITERSUMMARY, J_CONVEXIFYING,
 		     "Couenne: %d initial row cuts\n", cs.sizeRowCuts ());
 
-    if (realOpt && // this is a good time to check if we have messed up with the optimal solution
+    if (realOpt && // this is a good time to check if we have cut the optimal solution
 	isOptimumCut (realOpt, cs, problem_))
       jnlst_->Printf(J_ITERSUMMARY, J_CONVEXIFYING,
-		     "\n\n CUT OPTIMUM\n\n");
+		     "Warning: Optimal solution was cut\n");
   }
 
   catch (int exception) {
@@ -479,16 +611,7 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
       jnlst_ -> Printf (J_ITERSUMMARY, J_CONVEXIFYING,
 			"Couenne: Infeasible node\n");
 
-      OsiColCut *infeascut = new OsiColCut;
-
-      if (infeascut) {
-	int i=0;
-	double upper = -1., lower = +1.;
-	infeascut -> setLbs (1, &i, &lower);
-	infeascut -> setUbs (1, &i, &upper);
-	cs.insert (infeascut);
-	delete infeascut;
-      }
+      WipeMakeInfeas (cs);
     }
 
     if (babInfo) // set infeasibility to true in order to skip NLP heuristic
@@ -510,7 +633,9 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
     fictitiousBound (cs, problem_, true);
     firstcall_  = false;
     ntotalcuts_ = nrootcuts_ = cs.sizeRowCuts ();
+
   } else { 
+
     problem_ -> domain () -> pop ();
 
     ntotalcuts_ += (cs.sizeRowCuts () - nInitCuts);
@@ -533,3 +658,5 @@ void CouenneCutGenerator::generateCuts (const OsiSolverInterface &si,
   if (!(info.inTree)) 
     rootTime_ = CoinCpuTime ();
 }
+
+}
