@@ -1,21 +1,22 @@
-/* $Id: exprMinMax.cpp 154 2009-06-16 18:52:53Z pbelotti $ */
+/* $Id: exprMinMax.cpp 490 2011-01-14 16:07:12Z pbelotti $ */
 /*
  * Name:    exprMinMax.cpp
  * Author:  Pietro Belotti
  * Purpose: definition of min and max operators
  *
  * (C) Carnegie-Mellon University, 2006. 
- * This file is licensed under the Common Public License (CPL)
+ * This file is licensed under the Eclipse Public License (EPL)
  */
 
 #include "OsiSolverInterface.hpp"
 
 #include "CouenneCutGenerator.hpp"
 #include "CouenneTypes.hpp"
-#include "exprMax.hpp"
-#include "exprMin.hpp"
-#include "exprConst.hpp"
+#include "CouenneExprMax.hpp"
+#include "CouenneExprMin.hpp"
+#include "CouenneExprConst.hpp"
 
+using namespace Couenne;
 
 // Get lower and upper bound of an expression (if any)
 void exprMin::getBounds (expression *&lower, expression *&upper) {
@@ -31,14 +32,14 @@ void exprMax::getBounds (expression *&lower, expression *&upper) {
 }
 
 
-void exprMin::generateCuts (expression *w, const OsiSolverInterface &si, 
+void exprMin::generateCuts (expression *w, //const OsiSolverInterface &si, 
 			    OsiCuts &cs, const CouenneCutGenerator *cg,
 			    t_chg_bounds *chg, int,
 			    CouNumber, CouNumber) 
 {}
 
 
-void exprMax::generateCuts (expression *w, const OsiSolverInterface &si, 
+void exprMax::generateCuts (expression *w, //const OsiSolverInterface &si, 
 			    OsiCuts &cs, const CouenneCutGenerator *cg,
 			    t_chg_bounds *chg, int,
 			    CouNumber, CouNumber) 
